@@ -1,4 +1,4 @@
-__version__ = '0.3.1'
+__version__ = '0.3.12'
 
 __authors__ = [
     ("Thomas G. Close", "tom.g.close@gmail.com"),
@@ -6,6 +6,7 @@ __authors__ = [
     ("Phillip G. D. Ward", "phillipgdward@gmail.com")]
 
 
-install_requires = ['arcana>=0.4.1',
+install_requires = ['arcana>=0.4.12',
                     'pydicom>=1.0',
-                    'nibabel>=2.3.0']
+                    'nibabel>=2.3.0',
+                    'pybids>=0.9.1']
