@@ -26,14 +26,14 @@ os.environ['FSLOUTPUTTYPE'] = 'NIFTI_GZ'
 # Import all objects from Arcana used to design and apply Banana studies
 from arcana import (  # @IgnorePep8
     SubStudySpec, Parameter, ParamSpec, SwitchSpec, FileFormat, Fileset,
-    FilesetSpec, FilesetInput, FilesetInputSpec, FilesetCollection, Field,
-    FieldSpec, FieldInput, FieldInputSpec, FieldCollection, SingleProc,
-    MultiProc, SlurmProc, DEFAULT_PROV_CHECK, DEFAULT_PROV_IGNORE, StaticEnv,
+    FilesetSpec, InputFilesets, InputFilesetSpec, FilesetCollection, Field,
+    FieldSpec, InputFields, InputFieldSpec, FieldCollection, SingleProc,
+    MultiProc, SlurmProc, StaticEnv,
     ModulesEnv, BasicRepo, XnatRepo)
 
-from .bids import BidsRepo  # @IgnorePep8
+from .bids_ import BidsRepo  # @IgnorePep8
 
 
 # Import all Study classes into package root
-# from .study.base import (  # @IgnorePep8
-#     Study, StudyMetaClass, MultiStudy, MultiStudyMetaClass)
+from .study.base import (  # @IgnorePep8
+    Study, StudyMetaClass, MultiStudy, MultiStudyMetaClass)
