@@ -15,7 +15,7 @@ import nibabel
 # Import base file formats from Arcana for convenience
 from arcana.data.file_format import (
     text_format, directory_format, zip_format, targz_format,  # @UnusedImport
-    png_format, jpg_format, gif_format,  # @UnusedImport
+    png_format, jpg_format, gif_format, json_format,  # @UnusedImport
     UnzipConverter, UnTarGzConverter, IdentityConverter)  # @UnusedImport
 
 
@@ -204,7 +204,7 @@ class DicomFormat(ImageFormat):
                         dtype=int)
 
     def extract_id(self, fileset):
-        return int(fileset.dicom_values([self.SERIES_NUMBER_TAG]))
+        return int(fileset.dicom_values([self.SERIES_NUMBER_TAG])[0])
 
     def dicom_values(self, fileset, tags):
         """
