@@ -6,7 +6,7 @@ from nipype.interfaces.fsl.utils import Merge as fsl_merge
 from nipype.interfaces.fsl.epi import PrepareFieldmap
 from nipype.interfaces.fsl.preprocess import BET, FUGUE
 from arcana.study import ParamSpec, SwitchSpec
-from arcana.data import FilesetInputSpec, FilesetSpec, FieldSpec
+from arcana.data import InputFilesetSpec, FilesetSpec, FieldSpec
 from arcana.study.base import StudyMetaClass
 from banana.citation import fsl_cite
 from banana.requirement import fsl_req
@@ -15,23 +15,33 @@ from banana.interfaces.custom.motion_correction import (
 from banana.file_format import (
     nifti_gz_format, text_matrix_format,
     par_format, motion_mats_format, dicom_format)
-from banana.interfaces.custom.fmri import FieldMapTimeInfo
+from banana.interfaces.custom.bold import FieldMapTimeInfo
 from banana.interfaces.custom.motion_correction import (
     MergeListMotionMat, MotionMatCalculation)
-
+from banana.exceptions import BananaUsageError
 from banana.file_format import STD_IMAGE_FORMATS
+from banana.interfaces.mrtrix import MRConvert
+from banana.requirement import mrtrix_req
 
 
-class EpiStudy(MriStudy, metaclass=StudyMetaClass):
+class EpiSeriesStudy(MriStudy, metaclass=StudyMetaClass):
 
     add_data_specs = [
-        FilesetInputSpec('coreg_ref_wmseg', STD_IMAGE_FORMATS,
-                            optional=True),
-        FilesetInputSpec('reverse_phase', STD_IMAGE_FORMATS, optional=True),
-        FilesetInputSpec('field_map_mag', STD_IMAGE_FORMATS,
-                            optional=True),
-        FilesetInputSpec('field_map_phase', STD_IMAGE_FORMATS,
-                            optional=True),
+        InputFilesetSpec('series', STD_IMAGE_FORMATS,
+                         desc=("The set of EPI volumes that make up the "
+                               "series")),
+        InputFilesetSpec('coreg_ref_wmseg', STD_IMAGE_FORMATS,
+                         optional=True),
+        InputFilesetSpec('reverse_phase', STD_IMAGE_FORMATS, optional=True),
+        InputFilesetSpec('field_map_mag', STD_IMAGE_FORMATS,
+                         optional=True),
+        InputFilesetSpec('field_map_phase', STD_IMAGE_FORMATS,
+                         optional=True),
+        FilesetSpec('magnitude', nifti_gz_format, 'extract_magnitude_pipeline',
+                    desc=("The magnitude image, typically extracted from "
+                          "the provided series")),
+        FilesetSpec('series_preproc', nifti_gz_format, 'preprocess_pipeline'),
+        FilesetSpec('series_coreg', nifti_gz_format, 'series_coreg_pipeline'),
         FilesetSpec('moco', nifti_gz_format,
                     'intrascan_alignment_pipeline'),
         FilesetSpec('align_mats', motion_mats_format,
@@ -43,19 +53,34 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
 
     add_param_specs = [
         SwitchSpec('bet_robust', True),
-        SwitchSpec('linear_coreg_method', 'epireg',
-                   (MriStudy.parameter_spec('linear_coreg_method').choices +
-                    ('epireg',))),
+        MriStudy.param_spec('coreg_method').with_new_choices(
+            'epireg', fallbacks={'epireg': 'flirt'}),
         ParamSpec('bet_f_threshold', 0.2),
         ParamSpec('bet_reduce_bias', False),
         ParamSpec('fugue_echo_spacing', 0.000275)]
 
-    def linear_coreg_pipeline(self, **name_maps):
-        if self.branch('linear_coreg_method', 'epireg'):
-            return self._epireg_linear_coreg_pipeline(**name_maps)
+    @property
+    def header_image_spec_name(self):
+        if self.provided('header_image'):
+            hdr_name = 'header_image'
         else:
-            return super(EpiStudy, self).linear_brain_coreg_pipeline(
-                **name_maps)
+            hdr_name = 'series'
+        return hdr_name
+
+    @property
+    def series_preproc_spec_name(self):
+        if self.is_coregistered:
+            preproc = 'series_coreg'
+        else:
+            preproc = 'series_preproc'
+        return preproc
+
+    def coreg_pipeline(self, **name_maps):
+        if self.branch('coreg_method', 'epireg'):
+            pipeline = self._epireg_linear_coreg_pipeline(**name_maps)
+        else:
+            pipeline = super().coreg_pipeline(**name_maps)
+        return pipeline
 
     def _epireg_linear_coreg_pipeline(self, **name_maps):
 
@@ -66,20 +91,108 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
             citations=[fsl_cite],
             name_maps=name_maps)
 
-        pipeline.add(
+        epireg = pipeline.add(
             'epireg',
             fsl.epi.EpiReg(
-                out_base='epireg2ref'),
+                out_base='epireg2ref',
+                output_type='NIFTI_GZ'),
             inputs={
                 'epi': ('brain', nifti_gz_format),
                 't1_brain': ('coreg_ref_brain', nifti_gz_format),
-                't1_head': ('coreg_ref', nifti_gz_format),
-                'wmseg': ('coreg_ref_wmseg', nifti_gz_format)},
+                't1_head': ('coreg_ref', nifti_gz_format)},
             outputs={
-                'coreg_brain': ('out_file', nifti_gz_format),
-                'coreg_matrix': ('epi2str_mat', text_matrix_format)},
+                'brain_coreg': ('out_file', nifti_gz_format),
+                'coreg_fsl_mat': ('epi2str_mat', text_matrix_format)},
             requirements=[fsl_req.v('5.0.9')])
 
+        if self.provided('coreg_ref_wmseg'):
+            pipeline.connect_input('coreg_ref_wmseg', epireg, 'wmseg',
+                                   nifti_gz_format)
+
+        return pipeline
+
+    def brain_coreg_pipeline(self, **name_maps):
+        if self.branch('coreg_method', 'epireg'):
+            pipeline = self.coreg_pipeline(
+                name='brain_coreg',
+                name_maps=dict(
+                    input_map={
+                        'mag_preproc': 'brain',
+                        'coreg_ref': 'coreg_ref_brain'},
+                    output_map={
+                        'mag_coreg': 'brain_coreg'},
+                    name_maps=name_maps))
+
+            pipeline.add(
+                'mask_transform',
+                fsl.ApplyXFM(
+                    output_type='NIFTI_GZ',
+                    apply_xfm=True),
+                inputs={
+                    'in_matrix_file': (pipeline.node('epireg'), 'epi2str_mat'),
+                    'in_file': ('brain_mask', nifti_gz_format),
+                    'reference': ('coreg_ref_brain', nifti_gz_format)},
+                outputs={
+                    'brain_mask_coreg': ('out_file', nifti_gz_format)},
+                requirements=[fsl_req.v('5.0.10')],
+                wall_time=10)
+        else:
+            pipeline = super().coreg_brain_pipeline(**name_maps)
+
+        return pipeline
+
+    def extract_magnitude_pipeline(self, **name_maps):
+
+        pipeline = self.new_pipeline(
+            'extract_magnitude',
+            desc="Extracts a single magnitude volume from a series",
+            citations=[],
+            name_maps=name_maps)
+
+        pipeline.add(
+            "extract_first_vol",
+            MRConvert(
+                coord=(3, 0)),
+            inputs={
+                'in_file': ('series', nifti_gz_format)},
+            outputs={
+                'magnitude': ('out_file', nifti_gz_format)},
+            requirements=[mrtrix_req.v('3.0rc3')])
+
+        return pipeline
+
+    def series_coreg_pipeline(self, **name_maps):
+
+        pipeline = self.new_pipeline(
+            'series_coreg',
+            desc="Applies coregistration transform to DW series",
+            citations=[],
+            name_maps=name_maps)
+
+        if self.provided('coreg_ref'):
+            coreg_ref = 'coreg_ref'
+        elif self.provided('coreg_ref_brain'):
+            coreg_ref = 'coreg_ref_brain'
+        else:
+            raise BananaUsageError(
+                "Cannot coregister DW series as reference ('coreg_ref' or "
+                "'coreg_ref_brain') has not been provided to {}".format(self))
+
+        # Apply co-registration transformation to DW series
+        pipeline.add(
+            'mask_transform',
+            fsl.ApplyXFM(
+                output_type='NIFTI_GZ',
+                apply_xfm=True),
+            inputs={
+                'in_matrix_file': ('coreg_fsl_mat', text_matrix_format),
+                'in_file': ('series_preproc', nifti_gz_format),
+                'reference': (coreg_ref, nifti_gz_format)},
+            outputs={
+                'series_coreg': ('out_file', nifti_gz_format)},
+            requirements=[fsl_req.v('5.0.10')],
+            wall_time=10)
+
         return pipeline
 
     def intrascan_alignment_pipeline(self, **name_maps):
@@ -99,7 +212,7 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
                 output_type='NIFTI_GZ',
                 out_file='moco.nii.gz'),
             inputs={
-                'in_file': ('preproc', nifti_gz_format)},
+                'in_file': ('mag_preproc', nifti_gz_format)},
             outputs={
                 'moco': ('out_file', nifti_gz_format),
                 'moco_par': ('par_file', par_format)},
@@ -142,7 +255,7 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
         elif 'reverse_phase' in self.input_names:
             return self._topup_pipeline(**name_maps)
         else:
-            return super(EpiStudy, self).preprocess_pipeline(**name_maps)
+            return super().preprocess_pipeline(**name_maps)
 
     def _topup_pipeline(self, **name_maps):
 
@@ -156,7 +269,7 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
             'reorient_epi_in',
             fsl.utils.Reorient2Std(),
             inputs={
-                'in_file': ('magnitude', nifti_gz_format)},
+                'in_file': ('series', nifti_gz_format)},
             requirements=[fsl_req.v('5.0.9')])
 
         reorient_epi_opposite = pipeline.add(
@@ -192,14 +305,16 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
         merge = pipeline.add(
             'fsl_merge',
             fsl_merge(
-                dimension='t'),
+                dimension='t',
+                output_type='NIFTI_GZ'),
             inputs={
                 'in_files': (merge_outputs, 'out')},
             requirements=[fsl_req.v('5.0.9')])
 
         topup = pipeline.add(
             'topup',
-            TOPUP(),
+            TOPUP(
+                output_type='NIFTI_GZ'),
             inputs={
                 'in_file': (merge, 'merged_file'),
                 'encoding_file': (ped, 'config_file')},
@@ -215,14 +330,15 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
             'applytopup',
             ApplyTOPUP(
                 method='jac',
-                in_index=[1]),
+                in_index=[1],
+                output_type='NIFTI_GZ'),
             inputs={
                 'in_files': (in_apply_tp, 'out'),
                 'encoding_file': (ped, 'apply_topup_config'),
                 'in_topup_movpar': (topup, 'out_movpar'),
                 'in_topup_fieldcoef': (topup, 'out_fieldcoef')},
             outputs={
-                'preproc': ('out_corrected', nifti_gz_format)},
+                'series_preproc': ('out_corrected', nifti_gz_format)},
             requirements=[fsl_req.v('5.0.9')])
 
         return pipeline
@@ -237,21 +353,24 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
 
         reorient_epi_in = pipeline.add(
             'reorient_epi_in',
-            fsl.utils.Reorient2Std(),
+            fsl.utils.Reorient2Std(
+                output_type='NIFTI_GZ'),
             inputs={
-                'in_file': ('magnitude', nifti_gz_format)},
+                'in_file': ('series', nifti_gz_format)},
             requirements=[fsl_req.v('5.0.9')])
 
         fm_mag_reorient = pipeline.add(
             'reorient_fm_mag',
-            fsl.utils.Reorient2Std(),
+            fsl.utils.Reorient2Std(
+                output_type='NIFTI_GZ'),
             inputs={
                 'in_file': ('field_map_mag', nifti_gz_format)},
             requirements=[fsl_req.v('5.0.9')])
 
         fm_phase_reorient = pipeline.add(
             'reorient_fm_phase',
-            fsl.utils.Reorient2Std(),
+            fsl.utils.Reorient2Std(
+                output_type='NIFTI_GZ'),
             inputs={
                 'in_file': ('field_map_phase', nifti_gz_format)},
             requirements=[fsl_req.v('5.0.9')])
@@ -259,7 +378,8 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
         bet = pipeline.add(
             "bet",
             BET(
-                robust=True),
+                robust=True,
+                output_type='NIFTI_GZ'),
             inputs={
                 'in_file': (fm_mag_reorient, 'out_file')},
             wall_time=5,
@@ -282,12 +402,13 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
             FUGUE(
                 unwarp_direction='x',
                 dwell_time=self.parameter('fugue_echo_spacing'),
-                unwarped_file='example_func.nii.gz'),
+                unwarped_file='example_func.nii.gz',
+                output_type='NIFTI_GZ'),
             inputs={
                 'fmap_in_file': (create_fmap, 'out_fieldmap'),
                 'in_file': (reorient_epi_in, 'out_file')},
             outputs={
-                'preproc': ('unwarped_file', nifti_gz_format)},
+                'series_preproc': ('unwarped_file', nifti_gz_format)},
             wall_time=5,
             requirements=[fsl_req.v('5.0.9')])
 
@@ -305,7 +426,7 @@ class EpiStudy(MriStudy, metaclass=StudyMetaClass):
             'motion_mats',
             MotionMatCalculation(),
             inputs={
-                'reg_mat': ('coreg_matrix', text_matrix_format),
+                'reg_mat': ('coreg_fsl_mat', text_matrix_format),
                 'qform_mat': ('qform_mat', text_matrix_format)},
             outputs={
                 'motion_mats': ('motion_mats', motion_mats_format)})
