@@ -1,11 +1,12 @@
 from .base import MriStudy
 from nipype.interfaces.utility import Split
-from arcana import ParamSpec, FilesetSpec, StudyMetaClass
+from arcana import ParamSpec, SwitchSpec, FilesetSpec, StudyMetaClass
 from banana.exceptions import BananaUsageError
 from nipype.interfaces import fsl
 from banana.requirement import fsl_req
 from banana.citation import fsl_cite
-from banana.file_format import nifti_gz_format
+from banana.file_format import nifti_gz_format, nifti_gz_x_format
+from banana.bids_ import BidsInputs
 
 
 class T2Study(MriStudy, metaclass=StudyMetaClass):
@@ -14,10 +15,14 @@ class T2Study(MriStudy, metaclass=StudyMetaClass):
         FilesetSpec('wm_seg', nifti_gz_format, 'segmentation_pipeline')]
 
     add_param_specs = [
-        ParamSpec('bet_robust', True),
+        SwitchSpec('bet_robust', True),
         ParamSpec('bet_f_threshold', 0.5),
         ParamSpec('bet_reduce_bias', False)]
 
+    default_bids_inputs = [
+        BidsInputs(spec_name='magnitude', type='T2w',
+                   valid_formats=(nifti_gz_x_format, nifti_gz_format))]
+
     def segmentation_pipeline(self, img_type=2, **name_maps):
 
         pipeline = self.new_pipeline(
@@ -31,7 +36,8 @@ class T2Study(MriStudy, metaclass=StudyMetaClass):
             fsl.FAST(
                 img_type=img_type,
                 segments=True,
-                out_basename='Reference_segmentation'),
+                out_basename='Reference_segmentation',
+                output_type='NIFTI_GZ'),
             inputs={
                 'in_files': ('brain', nifti_gz_format)},
             requirements=[fsl_req.v('5.0.9')])
